@@ -5,6 +5,7 @@ import websocket
 import json
 import time
 import threading
+import logging
 
 
 @dataclass
@@ -18,8 +19,9 @@ class Metadata:
 
 class WebSocketStream:
 
-    def __init__(self, metadata: Metadata, on_property_changed_callback):
-        self.metadata = metadata
+    def __init__(self, webthing_property, on_property_changed_callback):
+        self.logger = logging.getLogger(webthing_property.metadata.name)
+        self.webthing_property = webthing_property
         self.on_property_changed_callback = on_property_changed_callback
         self.is_running = True
         self.thread = None
@@ -33,21 +35,24 @@ class WebSocketStream:
             try:
                 ws = websocket.WebSocket()
                 try:
-                    ws.connect(self.metadata.prop_ws_uri)
-                    print('webthing property ' + self.metadata.name + ' websocket ' + self.metadata.prop_ws_uri + ' connected')
+                    ws.connect(self.webthing_property.metadata.prop_ws_uri)
+                    self.logger.info('websocket ' + self.webthing_property.metadata.prop_ws_uri + ' connected')
+
+                    # set initial state (if changes are coming sporadically)
+                    self.on_property_changed_callback(self.webthing_property.property)
+
                     while self.is_running:
                         msg = json.loads(ws.recv())
                         if msg['messageType'] == 'propertyStatus':
                             data = msg['data']
-                            if self.metadata.name in data.keys():
-                                value = data[self.metadata.name]
-                                print("webthing property " + self.metadata.name + " has been updated to " + str(value))
+                            if self.webthing_property.metadata.name in data.keys():
+                                value = data[self.webthing_property.metadata.name]
                                 self.on_property_changed_callback(value)
                 finally:
-                    print('websocket ' + self.metadata.prop_ws_uri + ' disconnected')
+                    self.logger.info('websocket ' + self.webthing_property.metadata.prop_ws_uri + ' disconnected')
                     ws.close()
             except Exception as e:
-                print("error occurred consuming web socket for " + self.metadata.name  + " (" + self.metadata.prop_ws_uri + ") " + str(e))
+                self.logger.error("error occurred consuming web socket for " + self.webthing_property.metadata.name  + " (" + self.webthing_property.metadata.prop_ws_uri + ") " + str(e))
                 time.sleep(5)
 
     def stop(self):
@@ -58,14 +63,14 @@ class WebSocketStream:
 class WebthingProperty:
 
     def __init__(self, webthing_uri: str, webthing_property: str):
+        self.logger = logging.getLogger(webthing_property)
         self.__webthing_uri = webthing_uri
         self.__webthing_property = webthing_property
         self.__metadata = None
-        print("webthing property proxy created for " + self.__webthing_property + " (webthing uri " + self.__webthing_uri + ")")
 
+    @property
     def metadata(self) -> Metadata:
         if self.__metadata is None:
-            print('webthing property ' + self.__webthing_property + ' fetching meta data')
             response = requests.get(self.__webthing_uri)
             webthing_meta = response.json()
             props = webthing_meta['properties'][self.__webthing_property]
@@ -88,40 +93,26 @@ class WebthingProperty:
                         webthing_prop_ws_uri = urljoin(self.__webthing_uri, link['href'])
             self.__metadata = Metadata(self.__webthing_property, webthing_type, webthing_readonly, webthing_prop_uri, webthing_prop_ws_uri)
             response.close()
-            print('webthing property ' + self.__webthing_property + " meta data loaded (type: " + self.__metadata.type + ", readonly: " + str(self.__metadata.readonly) + ")")
+            self.logger.info("meta data loaded: " + str(self.__metadata))
         return self.__metadata
 
-    @property
-    def name(self) -> str:
-        return self.metadata().name
-
-    @property
-    def type(self) -> str:
-        return self.metadata().type
-
-    @property
-    def writeable(self) -> bool:
-        return not self.metadata().readonly
-
     @property
     def property(self):
-        response = requests.get(self.metadata().prop_uri)
+        response = requests.get(self.metadata.prop_uri)
         properties = response.json()
-        value =  properties[self.metadata().name]
+        value =  properties[self.metadata.name]
         response.close()
-        print("webthing property " + self.metadata().name + " read " + str(value))
         return value
 
     @property.setter
     def property(self, value):
         try:
-            print("writing webthing property " + self.name + " with " + str(value))
-            body = json.dumps({ self.metadata().name: value }, indent=2)
-            resp = requests.put(self.metadata().prop_uri, data=body, headers={'Content-Type': 'application/json'})
+            body = json.dumps({ self.metadata.name: value }, indent=2)
+            resp = requests.put(self.metadata.prop_uri, data=body, headers={'Content-Type': 'application/json'})
             resp.raise_for_status()
             resp.close()
         except requests.exceptions.HTTPError as err:
-            print("got error by writing webthing property " + self.name + " = " + str(value) + " using " + self.metadata().prop_uri + " reason: " + resp.text)
+            self.logger.error("got error by writing " + str(value) + " using " + self.metadata.prop_uri + " reason: " + resp.text)
 
     def new_change_listener(self, on_changed_callback) -> WebSocketStream:
-        return WebSocketStream(self.metadata(), on_changed_callback)
+        return WebSocketStream(self, on_changed_callback)
